@@ -10,6 +10,7 @@ import time
 import h5py
 import keras
 import keras.backend as K
+import tensorflow as tf
 import numpy as np
 from keras.callbacks import CSVLogger
 from keras.callbacks import LambdaCallback
@@ -146,12 +147,28 @@ class Janggu(object):
         else:
             total_output = K.sum(self.kerasmodel.output, axis=-1)
 
-        grad = K.gradients(total_output, self.kerasmodel.input)
         kinp = self.kerasmodel.input
-
         kinp = _to_list(kinp)
 
-        self._influence = K.function(kinp, grad)
+        if tf.__version__[0] == '1':
+            grad = K.gradients(total_output, self.kerasmodel.input)
+
+            self._influence = K.function(kinp, grad)
+        else:
+            def _influence(inputs):
+                tfinput = [tf.cast(inp, tf.float32) for inp in inputs]
+                with tf.GradientTape() as tape:
+                    tape.watch(tfinput)
+                    pred = self.kerasmodel(tfinput)
+                    if hasattr(outputs, '__len__') > 1:
+                        total_output = K.sum([K.sum(o, axis=-1) for o in pred], axis=-1)
+                    else:
+                        total_output = K.sum(pred, axis=-1)
+
+                grad = tape.gradient(total_output, tfinput)
+                return [g.numpy() for g in grad]
+
+            self._influence = _influence
 
         self.name = name
 
@@ -353,9 +370,9 @@ class Janggu(object):
         model = create_model(template, modelparams, inputs, outputs)
         return cls(model.inputs, model.outputs, name)
 
-    def compile(self, optimizer, loss, metrics=None,
-                loss_weights=None, sample_weight_mode=None,
-                weighted_metrics=None, target_tensors=None):
+    def compile(self, *args, **kargs):
+                #loss_weights=None, sample_weight_mode=None,
+                #weighted_metrics=None, target_tensors=None):
         """Model compilation.
 
         This method delegates the compilation to
@@ -371,9 +388,7 @@ class Janggu(object):
 
         """
 
-        self.kerasmodel.compile(optimizer, loss, metrics, loss_weights,
-                                sample_weight_mode, weighted_metrics,
-                                target_tensors)
+        self.kerasmodel.compile(*args, **kargs)
 
     def fit(self,  # pylint: disable=too-many-locals
             inputs=None,
@@ -865,9 +880,15 @@ class Janggu(object):
         if batch_size is None:
             batch_size = 128
 
+        def _get_input_layer_shape(layer):
+            if isinstance(layer.input_shape, list):
+                return layer.input_shape[0]
+            return layer.input_shape
+
         if len(self.kerasmodel.inputs) > 1:
             raise ValueError('Only one input layer supported for predict_variant_effect.')
-        binsize = self.kerasmodel.layers[0].input_shape[1] + bioseq.garray.order - 1
+        binsize = _get_input_layer_shape(self.kerasmodel.layers[0])[1] + \
+             bioseq.garray.order - 1
 
         if not bioseq.garray._full_genome_stored:
             raise ValueError('Incompatible Bioseq: '
