@@ -14,6 +14,7 @@ from keras.layers import Flatten
 from keras.layers import Input
 from keras.layers import Maximum
 from keras.layers import MaxPooling2D
+from keras import Model
 
 from janggu import Janggu
 from janggu import input_attribution
@@ -93,6 +94,62 @@ def test_janggu_generate_name(tmpdir):
     Janggu.create_by_name(bwm.name)
 
 
+def test_dnaconv():
+    data_path = pkg_resources.resource_filename('janggu', 'resources/')
+    bed_file = os.path.join(data_path, 'sample.bed')
+
+    refgenome = os.path.join(data_path, 'sample_genome.fa')
+
+    dna = Bioseq.create_from_refgenome('dna', refgenome=refgenome,
+                                    storage='ndarray',
+                                    roi=bed_file, order=1)
+
+    xin = Input(dna.shape[1:])
+    l1 = DnaConv2D(Conv2D(30, (21, 1), activation='relu'))(xin)
+    m1 = Model(xin, l1)
+    res1 =m1.predict(dna[0])[0,0,0,:]
+
+    clayer = m1.layers[1].forward_layer
+    # forward only
+    l1 = clayer(xin)
+    m2 = Model(xin, l1)
+    res2 = m2.predict(dna[0])[0,0, 0,:]
+
+    rxin = Reverse()(Complement()(xin))
+    l1 = clayer(rxin)
+    l1 = Reverse()(l1)
+    m3 = Model(xin, l1)
+    res3 = m3.predict(dna[0])[0,0, 0,:]
+
+    res4 = np.maximum(res3,res2)
+    np.testing.assert_allclose(res1, res4, rtol=1e-4)
+
+
+def test_dnaconv2():
+    # this checks if DnaConv2D layer is instantiated correctly if
+    # the conv2d layer has been instantiated beforehand.
+    data_path = pkg_resources.resource_filename('janggu', 'resources/')
+    bed_file = os.path.join(data_path, 'sample.bed')
+
+    refgenome = os.path.join(data_path, 'sample_genome.fa')
+
+    dna = Bioseq.create_from_refgenome('dna', refgenome=refgenome,
+                                    storage='ndarray',
+                                    roi=bed_file, order=1)
+
+    xin = Input(dna.shape[1:])
+    clayer = Conv2D(30, (21, 1), activation='relu')
+
+    clayer(xin)
+
+    l1 = DnaConv2D(clayer)(xin)
+    m1 = Model(xin, l1)
+    res1 =m1.predict(dna[0])[0,0,0,:]
+
+    np.testing.assert_allclose(clayer.get_weights()[0], m1.layers[1].forward_layer.get_weights()[0])
+    assert len(clayer.weights) == 2
+
+
 @pytest.mark.filterwarnings("ignore:The truth value")
 def test_janggu_instance_dense(tmpdir):
     os.environ['JANGGU_OUTPUT'] = tmpdir.strpath
@@ -1036,6 +1093,41 @@ def test_janggu_train_predict_option7(tmpdir):
     bwm.evaluate(inputs, outputs,
                  use_multiprocessing=False)
 
+@pytest.mark.filterwarnings("ignore:inspect")
+def test_sequence_config():
+    """Train, predict and evaluate on dummy data.
+
+    create: YES
+    Input args: Dataset
+    validation_set: YES
+    batch_size: None
+    """
+
+    inputs = Array("x", np.random.random((100, 10)))
+    outputs = Array('y', np.random.randint(2, size=(100, 1)),
+                    conditions=['random'])
+
+    jseq = JangguSequence(inputs.data, outputs.data, batch_size=10, as_dict=False)
+    assert len(jseq) == 10
+    for x, y, _ in jseq:
+        assert x[0].shape == (10, 10)
+        assert y[0].shape == (10, 1)
+        break
+
+    jseq = JangguSequence(inputs, outputs, batch_size=10, as_dict=False)
+    assert len(jseq) == 10
+    for x, y, _ in jseq:
+        assert x[0].shape == (10, 10)
+        assert y[0].shape == (10, 1)
+        break
+
+    jseq = JangguSequence(inputs, outputs, batch_size=10, as_dict=True)
+    assert len(jseq) == 10
+    for x, y, _ in jseq:
+        assert x['x'].shape == (10, 10)
+        assert y['y'].shape == (10, 1)
+        break
+
 
 @pytest.mark.filterwarnings("ignore:inspect")
 def test_janggu_train_predict_sequence(tmpdir):
